@@ -1,11 +1,11 @@
-use either::Either;
 use hir::{HasSource, InFile, Semantics};
 use ide_db::{
     base_db::{FileId, FilePosition, FileRange},
+    defs::Definition,
     helpers::visit_file_defs,
     RootDatabase,
 };
-use syntax::{ast::NameOwner, AstNode, TextRange};
+use syntax::{ast::HasName, AstNode, TextRange};
 
 use crate::{
     fn_references::find_all_methods,
@@ -40,6 +40,7 @@ pub struct AnnotationConfig {
     pub annotate_impls: bool,
     pub annotate_references: bool,
     pub annotate_method_references: bool,
+    pub annotate_enum_variant_references: bool,
 }
 
 pub(crate) fn annotations(
@@ -61,54 +62,87 @@ pub(crate) fn annotations(
         }
     }
 
-    visit_file_defs(&Semantics::new(db), file_id, &mut |def| match def {
-        Either::Left(def) => {
-            let range = match def {
-                hir::ModuleDef::Const(konst) => {
-                    konst.source(db).and_then(|node| name_range(&node, file_id))
+    visit_file_defs(&Semantics::new(db), file_id, &mut |def| {
+        let range = match def {
+            Definition::Const(konst) if config.annotate_references => {
+                konst.source(db).and_then(|node| name_range(db, node, file_id))
+            }
+            Definition::Trait(trait_) if config.annotate_references || config.annotate_impls => {
+                trait_.source(db).and_then(|node| name_range(db, node, file_id))
+            }
+            Definition::Adt(adt) => match adt {
+                hir::Adt::Enum(enum_) => {
+                    if config.annotate_enum_variant_references {
+                        enum_
+                            .variants(db)
+                            .into_iter()
+                            .map(|variant| {
+                                variant.source(db).and_then(|node| name_range(db, node, file_id))
+                            })
+                            .filter_map(std::convert::identity)
+                            .for_each(|range| {
+                                annotations.push(Annotation {
+                                    range,
+                                    kind: AnnotationKind::HasReferences {
+                                        position: FilePosition { file_id, offset: range.start() },
+                                        data: None,
+                                    },
+                                })
+                            })
+                    }
+                    if config.annotate_references || config.annotate_impls {
+                        enum_.source(db).and_then(|node| name_range(db, node, file_id))
+                    } else {
+                        None
+                    }
                 }
-                hir::ModuleDef::Trait(trait_) => {
-                    trait_.source(db).and_then(|node| name_range(&node, file_id))
+                _ => {
+                    if config.annotate_references || config.annotate_impls {
+                        adt.source(db).and_then(|node| name_range(db, node, file_id))
+                    } else {
+                        None
+                    }
                 }
-                hir::ModuleDef::Adt(adt) => {
-                    adt.source(db).and_then(|node| name_range(&node, file_id))
-                }
-                _ => None,
-            };
-            let (range, offset) = match range {
-                Some(range) => (range, range.start()),
-                None => return,
-            };
+            },
+            _ => None,
+        };
 
-            if config.annotate_impls && !matches!(def, hir::ModuleDef::Const(_)) {
-                annotations.push(Annotation {
-                    range,
-                    kind: AnnotationKind::HasImpls {
-                        position: FilePosition { file_id, offset },
-                        data: None,
-                    },
-                });
-            }
-            if config.annotate_references {
-                annotations.push(Annotation {
-                    range,
-                    kind: AnnotationKind::HasReferences {
-                        position: FilePosition { file_id, offset },
-                        data: None,
-                    },
-                });
-            }
+        let (range, offset) = match range {
+            Some(range) => (range, range.start()),
+            None => return,
+        };
 
-            fn name_range<T: NameOwner>(node: &InFile<T>, file_id: FileId) -> Option<TextRange> {
-                if node.file_id == file_id.into() {
-                    node.value.name().map(|it| it.syntax().text_range())
-                } else {
-                    // Node is outside the file we are adding annotations to (e.g. macros).
-                    None
+        if config.annotate_impls && !matches!(def, Definition::Const(_)) {
+            annotations.push(Annotation {
+                range,
+                kind: AnnotationKind::HasImpls {
+                    position: FilePosition { file_id, offset },
+                    data: None,
+                },
+            });
+        }
+        if config.annotate_references {
+            annotations.push(Annotation {
+                range,
+                kind: AnnotationKind::HasReferences {
+                    position: FilePosition { file_id, offset },
+                    data: None,
+                },
+            });
+        }
+
+        fn name_range<T: HasName>(
+            db: &RootDatabase,
+            node: InFile<T>,
+            source_file_id: FileId,
+        ) -> Option<TextRange> {
+            if let Some(InFile { file_id, value }) = node.original_ast_node(db) {
+                if file_id == source_file_id.into() {
+                    return value.name().map(|it| it.syntax().text_range());
                 }
             }
+            None
         }
-        Either::Right(_) => (),
     });
 
     if config.annotate_method_references {
@@ -173,6 +207,7 @@ mod tests {
                     annotate_impls: true,
                     annotate_references: true,
                     annotate_method_references: true,
+                    annotate_enum_variant_references: true,
                 },
                 file_id,
             )
